#!/bin/env -S deno run -A --unstable

if (import.meta.main) await main();
async function main() {
	const [from_port, to_ip, to_port] = Deno.args;
	if (to_port === undefined) {
		console.error("[proxy] usage: ./proxy.ts <from_port> <to_ip> <to_port>");
		Deno.exit(2);
	}

	const server = Deno.listen({ transport: "tcp", port: parseInt(from_port) });
	console.log("[proxy] listening on port", from_port, "redirecting to", to_ip, "port", to_port);

	for await (const connection of server) {
		serve(to_ip, to_port, connection);
	}
}

async function serve(to_ip: string, to_port: string, connection: Deno.Conn) {
	try {
		const client = await Deno.connect({ transport: "tcp", hostname: to_ip, port: parseInt(to_port) });
		const promise_connection_done = connection.readable.pipeTo(client.writable);
		const promise_client_done = client.readable.pipeTo(connection.writable);
		await Promise.all([promise_client_done, promise_connection_done]);
	} catch (_) { /* isok */ }
}

export function proxy_command(from_port: number, to_ip: string, to_port: number) {
	const path = new URL("", import.meta.url).pathname;
	const args = [from_port, to_ip, to_port].map((v) => v.toString());
	return new Deno.Command(path, { args });
}
